//---------------------------Rube Goldberg "Do Nothing Machine"-------------------------------------------------------
// This demo will show how to connect five basic Arduino tinker-toy-like parts into a more complex system
// It will use the following parts: ultrasonicRanger, IR_Obstacle_Sensor,Servo_Motor, piezo buzzer 
// and speaker
//-----------------------------BruceBullock July 2023------------------------------------------------

#include <TimerFreeTone.h>  //using instead of Tone to play tones to avoid timer conflict with NewPing
#include <Servo.h>          //This includes the external Servo motor library
Servo myservo; // create servo object to control a servo

#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27, 16, 2); // I2C address 0x27, 16 column and 2 rows

//Ultrasonic Range Sensor stuff
#include <NewPing.h> 
#define trig_pin A1 //analog input 1
#define echo_pin A2 //analog input 2
#define maximum_distance 200 //distance is in centimeters
NewPing sonar(trig_pin, echo_pin, maximum_distance); //Sonar sensor function

//Variable declarations
int distance = 0;
int observerStatus = 0;    //Stores state of Ultrasonic sensor, 0 until observer detected
int ballStatus =0;     //Stores state of ir ball sensor
int ballHit = 0;       //0 until ball hit, then 1
int count = 0;
int counter = 0;


int Buzzer = 7; // Piezo Buzzer
int speaker = 6;      // Small sound speaker
int irSensorPin = 9;  // connects IR sensor module to Arduino pin D9
// Servo is connected to 10 (see below)
// 11 is unused
int greenLED = 12;    // green LED
int redLED = 13;      // red LED 

/*-----------------------Constants used to Play Music Tone Notes on spraker----------------*/
const int c = 261;
const int d = 294;
const int e = 329;
const int f = 349;
const int g = 391;
const int gS = 415;
const int a = 440;
const int aS = 455;
const int b = 466;
const int cH = 523;
const int cSH = 554;
const int dH = 587;
const int dSH = 622;
const int eH = 659;
const int fH = 698;
const int fSH = 740;
const int gH = 784;
const int gSH = 830;
const int aH = 880;
 
//----------------Note: This implementation uses FUNCTIONS to simplify repeating structure----
//----------------Function Definitions Below---------------------------------------------------
void blink()                      //Function that blinks the LED on and then off for 1/2 second
  {
    digitalWrite(redLED, HIGH);  // turn the LED on (HIGH is the voltage level)
    delay(500);                      // wait for a second
    digitalWrite(redLED, LOW);   // turn the LED off by making the voltage LOW
    delay(500);                      // wait for a second
  }

void blink5times()              //Blinks LED 5 times
    {
      for (int i=0; i<5; i=i+1){
      digitalWrite(12, HIGH);  // turn the LED on
      delay(100);       // wait for a half second               
      digitalWrite(12, LOW);  // turn the LED off
      delay(100);       // wait for a half second
      }
    }

int readPing(){               //Function senses distance of observer using the ultrasonic distance sensor 
  delay(50);
  int cm = sonar.ping_cm();
  if (cm==0){
  cm=250;
  }
  //Serial.println("cm");     //These are test print lines that are left as comments for now
  //Serial.println(cm);
  return cm;                  //The function returns the distance in centimeters 
}

/* -------------------------Setup Code--------------------------------------------------------- */
void setup() // This sets up code that will only run once at the start
{                   
 Serial.begin(9600);             //This connects communication between the Arduino and the Laptop Serial Monitor
 myservo.attach(10);             // Attaches the servo on pin 10 to the servo
 pinMode(redLED, OUTPUT);        // The following sets pins as the digital output pins
 pinMode(greenLED, OUTPUT);
 pinMode(irSensorPin, INPUT);    
 pinMode(Buzzer, OUTPUT);
 pinMode(speaker, OUTPUT);
 
 lcd.init();                    // initialize the lcd display
 lcd.backlight();

 myservo.write(180);              //Sets the servo  back to be ready to bat ball when observer shows up
}

void loop()                       //Beginning of the repeating loop
{                
  //-------------This prints out an initial title while waiting for an observer to show up----
  lcd.setCursor(0, 0);           // move cursor to   (0, 0)
  lcd.print("Do Nothing Thing"); // print message at (0, 0) in top line, lines start at 0
  lcd.setCursor(4, 10);           // move cursor to   (2, 9)
  lcd.print("Machine");           // print message at (2, 1)
  
//----------------ULTRASONIC SENSOR CODE----------------------
    int observerStatus;     //State of Ultrasonic sensor
    distance = readPing();  //Start the loop by sensing if an observer is present

    if (distance < 75) 
      {        //Check for presence of a close observer closer than 75 cm
      observerStatus = 1;       //Set status to 1 when an observer is detected
        blink;
      //Indicate on display that an observer is present and at what distance          
      lcd.clear();              // clear display
      lcd.setCursor(0, 0);      // move cursor to   (0, 0)
      lcd.print("Observer at"); // print message at (0, 0)
      lcd.setCursor(13, 0);      // start to print at the first row
      lcd.print(distance);        // print the distance
  
      //Play Star Wars Song
      firstSection(); 

        //Hit the Ball
        myservo.write(150); //Touch ball to stop bouncing
        delay(600);
        myservo.write(180); //Move bat farther back
        delay(500);
        myservo.write(105); //Hit the ball
        delay(200);
        myservo.write(180); //Retreat

  count = 0;
      while (count < 2) {
      digitalWrite(Buzzer, HIGH);
      delay(200);
      digitalWrite(Buzzer, LOW);
      delay(200);
      count++;
      

      }
    }
  else
  {
  //------------------------This is the bottom of the detection loop--------------------
  blink();         //If no detection, then blink red LED and start detection loop over
  delay(300);
  }
}



void beep(int note, int duration)
{
  //Play tone on buzzerPin
  TimerFreeTone(speaker, note, duration); 
  //tone(speaker, note, duration); 
 
  //Play different LED depending on value of 'counter'
  if(counter % 2 == 0)
  {
    digitalWrite(redLED, HIGH);
    delay(duration);
    digitalWrite(redLED, LOW);
  }else
  {
    digitalWrite(greenLED, HIGH);
    delay(duration);
    digitalWrite(greenLED, LOW);
  }
  delay(50);
  //Increment counter
  counter++;
}
 
void firstSection()
{
  beep(a, 250);
  beep(a, 250);    
  beep(a, 250);
  beep(f, 125);
  beep(cH, 75);  
  beep(a, 250);
  beep(f, 175);
  beep(cH, 75);
  beep(a, 325);
 
  delay(500);
 
  beep(eH, 250);
  beep(eH, 250);
  beep(eH, 250);  
  beep(fH, 175);
  beep(cH, 75);
  beep(gS, 250);
  beep(f, 175);
  beep(cH, 75);
  beep(a, 325);
 
  delay(500);
}
